@@ -1,7 +1,7 @@
 /*
  * This file is part of the planetblupi source code
  * Copyright (C) 1997, Daniel Roux & EPSITEC SA
- * Copyright (C) 2017, Mathieu Schroeter
+ * Copyright (C) 2017-2018, Mathieu Schroeter
  * http://epsitec.ch; http://www.blupi.org; http://github.com/blupi-games
  *
  * This program is free software: you can redistribute it and/or modify
@@ -28,9 +28,10 @@
 #include <stdlib.h>
 #include <string>
 #include <thread>
+#include <unistd.h>
 #include <vector>
 
-#include <SDL2/SDL_image.h>
+#include <SDL_image.h>
 #include <argagg/argagg.hpp>
 #ifdef USE_CURL
 #include <curl/curl.h>
@@ -47,6 +48,7 @@
 #include "misc.h"
 #include "movie.h"
 #include "pixmap.h"
+#include "platform.h"
 #include "progress.h"
 #include "sound.h"
 
@@ -72,10 +74,12 @@ bool        g_enableRecorder = false;
 std::string g_playRecord;
 bool        g_restoreBugs = false; // restore original < v1.9 bugs
 bool        g_restoreMidi = false; // restore music playback based on midi files
+bool        g_renderQuality = true; // use the best render quality with scaling
 int         g_settingsOverload = 0;
 
 bool        g_bTermInit = false; // initialisation en cours
 Uint32      g_lastPhase = 999;
+RestartMode g_restart   = RestartMode::NO;
 static bool g_pause;
 
 #ifdef USE_CURL
@@ -84,7 +88,7 @@ struct url_data {
   char *   buffer;
   size_t   size;
 };
-#endif
+#endif /* USE_CURL */
 
 template <typename Out>
 static void
@@ -179,6 +183,11 @@ ReadConfig ()
     !(g_settingsOverload & SETTING_MIDI) && j.find ("restoremidi") != j.end ())
     g_restoreMidi = j["restoremidi"].get<bool> ();
 
+  if (
+    !(g_settingsOverload & SETTING_RENDERQUALITY) &&
+    j.find ("renderquality") != j.end ())
+    g_renderQuality = j["renderquality"].get<bool> ();
+
   return true;
 }
 
@@ -186,7 +195,7 @@ ReadConfig ()
  * \brief Main frame update.
  */
 static bool
-UpdateFrame (void)
+Update (void)
 {
   Rect   clip, rcRect;
   Uint32 phase;
@@ -202,8 +211,8 @@ UpdateFrame (void)
 
   rcRect.left   = 0;
   rcRect.top    = 0;
-  rcRect.right  = LXIMAGE;
-  rcRect.bottom = LYIMAGE;
+  rcRect.right  = LXIMAGE ();
+  rcRect.bottom = LYIMAGE ();
   g_pPixmap->DrawImage (-1, CHBACK, rcRect); // draw the background
 
   if (phase == EV_PHASE_INTRO1)
@@ -232,7 +241,8 @@ UpdateFrame (void)
 
     g_pEvent->DecorAutoShift ();
     g_pDecor->Build (clip, posMouse); // build the environment
-    g_pDecor->NextPhase (1);          // rebuild the map sometimes
+    g_pEvent->PlayMove (posMouse);
+    g_pDecor->NextPhase (1); // rebuild the map sometimes
   }
 
   if (phase == EV_PHASE_BUILD)
@@ -251,6 +261,13 @@ UpdateFrame (void)
     g_pEvent->DemoStep (); // start automatically (eventually) the demo
   }
 
+  SDL_Event event = {0};
+  event.type      = SDL_MOUSEMOTION;
+  event.motion.x  = posMouse.x;
+  event.motion.y  = posMouse.y;
+  g_pEvent->EventButtons (event, posMouse);
+  g_pEvent->MouseSprite (posMouse);
+
   g_pEvent->DrawButtons ();
 
   g_lastPhase = phase;
@@ -387,7 +404,9 @@ HandleEvent (const SDL_Event & event)
 
   case SDL_RENDER_DEVICE_RESET:
   case SDL_RENDER_TARGETS_RESET:
+    g_pDecor->InvalidateGrounds ();
     g_pPixmap->ReloadTargetTextures ();
+    g_pEvent->LoadBackground ();
     break;
 
   case SDL_USEREVENT:
@@ -400,7 +419,7 @@ HandleEvent (const SDL_Event & event)
         bool display = true;
 
         if (!g_pause)
-          display = UpdateFrame ();
+          display = Update ();
 
         if (!g_pEvent->IsMovie () && display)
           g_pPixmap->Display ();
@@ -410,7 +429,10 @@ HandleEvent (const SDL_Event & event)
     case EV_WARPMOUSE:
     {
       const SDL_Point * coord = static_cast<SDL_Point *> (event.user.data1);
-      SDL_WarpMouseInWindow (g_window, coord->x, coord->y);
+
+      Sint32 x = coord->x, y = coord->y;
+      g_pPixmap->FromGameToDisplay (x, y);
+      SDL_WarpMouseInWindow (g_window, x, y);
       delete coord;
       break;
     }
@@ -501,10 +523,10 @@ progressCallback (
 }
 #endif /* USE_CURL */
 
+#ifdef USE_CURL
 static void
 CheckForUpdates ()
 {
-#ifdef USE_CURL
   url_data chunk;
 
   chunk.buffer = nullptr; /* we expect realloc(NULL, size) to work */
@@ -548,8 +570,8 @@ CheckForUpdates ()
     free (chunk.buffer);
 
   curl_easy_cleanup (curl);
-#endif /* USE_CURL */
 }
+#endif /* USE_CURL */
 
 static int
 parseArgs (int argc, char * argv[], bool & exit)
@@ -573,6 +595,10 @@ parseArgs (int argc, char * argv[], bool & exit)
       {"-z", "--zoom"},
       "change the window scale (only if fullscreen is off) [1;2] (default: 1)",
       1},
+     {"legacy",
+      {"-l", "--legacy"},
+      "start the game in legacy display mode (640x480)",
+      0},
      {"renderer",
       {"-r", "--renderer"},
       "set a renderer [auto;software;accelerated] (default: auto)",
@@ -598,7 +624,12 @@ parseArgs (int argc, char * argv[], bool & exit)
      {"restoremidi",
       {"-m", "--restore-midi"},
       "restore playback based on MIDI music instead of OGG",
-      0}}};
+      0},
+     {"renderquality",
+      {"-q", "--render-quality"},
+      "enable anti-aliasing [on;off] (default: on, ignored if "
+      "windowed)",
+      1}}};
 
   argagg::parser_results args;
   try
@@ -653,6 +684,12 @@ parseArgs (int argc, char * argv[], bool & exit)
     g_settingsOverload |= SETTING_ZOOM;
   }
 
+  if (args["legacy"])
+  {
+    Display::getDisplay ().setDisplaySize (LXLOGIC (), LYLOGIC ());
+    g_settingsOverload |= SETTING_LEGACY;
+  }
+
   if (args["renderer"])
   {
     if (args["renderer"].as<std::string> () == "auto")
@@ -690,6 +727,13 @@ parseArgs (int argc, char * argv[], bool & exit)
     g_settingsOverload |= SETTING_MIDI;
   }
 
+  if (args["renderquality"])
+  {
+    g_renderQuality =
+      args["renderquality"].as<std::string> () != std::string ("off");
+    g_settingsOverload |= SETTING_RENDERQUALITY;
+  }
+
   return EXIT_SUCCESS;
 }
 
@@ -714,6 +758,15 @@ DoInit (int argc, char * argv[], bool & exit)
     return EXIT_FAILURE;
   }
 
+#ifdef __LINUX__
+  if (!getenv ("ALSA_CONFIG_DIR"))
+  {
+    static char env[256];
+    snprintf (env, sizeof (env), "ALSA_CONFIG_DIR=/usr/share/alsa");
+    putenv (env);
+  }
+#endif /* __LINUX__ */
+
 #ifdef _WIN32
   /* Fix laggy sounds on Windows by not using winmm driver. */
   SDL_setenv ("SDL_AUDIODRIVER", "directsound", true);
@@ -726,10 +779,13 @@ DoInit (int argc, char * argv[], bool & exit)
     return EXIT_FAILURE;
   }
 
+  if (!(g_settingsOverload & SETTING_LEGACY))
+    Display::getDisplay ().readDisplaySize ();
+
   // Create a window.
   g_window = SDL_CreateWindow (
     gettext ("Planet Blupi"), SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED,
-    LXIMAGE, LYIMAGE, 0);
+    LXIMAGE (), LYIMAGE (), 0);
 
   if (!g_window)
   {
@@ -752,7 +808,7 @@ DoInit (int argc, char * argv[], bool & exit)
     return EXIT_FAILURE;
   }
 
-  SDL_RenderSetLogicalSize (g_renderer, LXIMAGE, LYIMAGE);
+  SDL_RenderSetLogicalSize (g_renderer, LXIMAGE (), LYIMAGE ());
 
   const auto renders = SDL_GetNumRenderDrivers ();
   for (int i = 0; i < renders; ++i)
@@ -779,16 +835,24 @@ DoInit (int argc, char * argv[], bool & exit)
       info.max_texture_height);
   }
 
+  // Create the event manager.
+  g_pEvent = new CEvent;
+  if (g_pEvent == nullptr)
+  {
+    InitFail ("New event");
+    return EXIT_FAILURE;
+  }
+
   // Create the main pixmap.
-  g_pPixmap = new CPixmap;
+  g_pPixmap = new CPixmap (g_pEvent);
   if (g_pPixmap == nullptr)
   {
     InitFail ("New pixmap");
     return EXIT_FAILURE;
   }
 
-  totalDim.x = LXIMAGE;
-  totalDim.y = LYIMAGE;
+  totalDim.x = LXIMAGE ();
+  totalDim.y = LYIMAGE ();
   if (!g_pPixmap->Create (totalDim))
   {
     InitFail ("Create pixmap");
@@ -796,8 +860,8 @@ DoInit (int argc, char * argv[], bool & exit)
   }
 
   OutputDebug ("Image: init\n");
-  totalDim.x = LXIMAGE;
-  totalDim.y = LYIMAGE;
+  totalDim.x = LXLOGIC ();
+  totalDim.y = LYLOGIC ();
   iconDim.x  = 0;
   iconDim.y  = 0;
 #if _INTRO
@@ -808,17 +872,15 @@ DoInit (int argc, char * argv[], bool & exit)
     return EXIT_FAILURE;
 
   OutputDebug ("Image: init\n");
-  totalDim.x = LXIMAGE;
-  totalDim.y = LYIMAGE;
-  iconDim.x  = 0;
-  iconDim.y  = 0;
-  if (!g_pPixmap->Cache (CHGROUND, "image/init.png", totalDim, iconDim))
+  totalDim.x = LXIMAGE ();
+  totalDim.y = LYIMAGE ();
+  if (!g_pPixmap->Cache (CHGROUND, totalDim))
     return EXIT_FAILURE;
 
   rcRect.left   = 0;
   rcRect.top    = 0;
-  rcRect.right  = LXIMAGE;
-  rcRect.bottom = LYIMAGE;
+  rcRect.right  = LXIMAGE ();
+  rcRect.bottom = LYIMAGE ();
   g_pPixmap->DrawImage (-1, CHBACK, rcRect); // dessine le fond
   g_pPixmap->Display ();
 
@@ -961,7 +1023,7 @@ DoInit (int argc, char * argv[], bool & exit)
   g_pSound->SetState (true);
 
   // Create the movie manager.
-  g_pMovie = new CMovie;
+  g_pMovie = new CMovie (g_pPixmap);
   if (g_pMovie == nullptr)
   {
     InitFail ("New movie");
@@ -981,27 +1043,24 @@ DoInit (int argc, char * argv[], bool & exit)
   g_pDecor->Create (g_pSound, g_pPixmap);
   g_pDecor->MapInitColors ();
 
-  // Create the event manager.
-  g_pEvent = new CEvent;
-  if (g_pEvent == nullptr)
-  {
-    InitFail ("New event");
-    return EXIT_FAILURE;
-  }
-
   const bool zoom = g_zoom;
 
+  const bool renderQuality = g_renderQuality;
   g_pEvent->Create (g_pPixmap, g_pDecor, g_pSound, g_pMovie);
+  if (renderQuality != g_renderQuality)
+    g_pPixmap->CreateMainTexture ();
 
   // Load all cursors
-  g_pPixmap->LoadCursors (g_zoom);
+  g_pPixmap->LoadCursors ();
   g_pPixmap->ChangeSprite (SPRITE_WAIT);
 
+#ifdef USE_CURL
   g_updateThread = new std::thread (CheckForUpdates);
-  if (g_bFullScreen)
-    g_pEvent->SetFullScreen (true);
-  if (!g_bFullScreen && zoom != g_zoom)
+#endif /* USE_CURL */
+
+  if (zoom != g_zoom)
     g_pEvent->SetWindowSize (g_zoom);
+  g_pEvent->SetFullScreen (g_bFullScreen);
   g_pEvent->ChangePhase (EV_PHASE_INTRO1);
 
   g_bTermInit = true;
@@ -1027,23 +1086,8 @@ main (int argc, char * argv[])
   if ((res = DoInit (argc, argv, exit)) || exit)
     return res;
 
-  SDL_TimerID updateTimer = SDL_AddTimer (
-    g_timerInterval,
-    [](Uint32 interval, void * param) -> Uint32 {
-      CEvent::PushUserEvent (EV_UPDATE);
-      return interval;
-    },
-    nullptr);
+  Platform::run (HandleEvent);
 
-  SDL_Event event;
-  while (SDL_WaitEvent (&event))
-  {
-    HandleEvent (event);
-    if (event.type == SDL_QUIT)
-      break;
-  }
-
-  SDL_RemoveTimer (updateTimer);
   FinishObjects ();
 
   if (g_renderer)
@@ -1061,5 +1105,32 @@ main (int argc, char * argv[])
     delete (g_updateThread);
   }
 
+  /* Restart the game when the fullscreen mode (ratio) has changed. */
+  if (g_restart != RestartMode::NO)
+  {
+    std::vector<const char *> _argv;
+    std::string               argv0;
+    std::string               bin = basename (argv[0]);
+
+    if (getenv ("APPIMAGE"))
+      argv0 = getenv ("APPIMAGE");
+    else
+      argv0 = GetBinDir () + bin;
+
+    _argv.push_back (bin.c_str ());
+    if (g_restart == RestartMode::LEGACY)
+    {
+      _argv.push_back ("--legacy");
+      _argv.push_back ("--fullscreen");
+      _argv.push_back ("on");
+      _argv.push_back ("--zoom");
+      _argv.push_back ("2");
+    }
+    _argv.push_back (nullptr);
+
+    SDL_Log ("Reload the game from %s", argv0.c_str ());
+    execv (argv0.c_str (), const_cast<char **> (&_argv[0]));
+  }
+
   return 0;
 }
