@@ -1,7 +1,7 @@
 /*
  * This file is part of the planetblupi source code
  * Copyright (C) 1997, Daniel Roux & EPSITEC SA
- * Copyright (C) 2017, Mathieu Schroeter
+ * Copyright (C) 2017-2018, Mathieu Schroeter
  * http://epsitec.ch; http://www.blupi.org; http://github.com/blupi-games
  *
  * This program is free software: you can redistribute it and/or modify
@@ -274,8 +274,8 @@ CButton::TreatEvent (const SDL_Event & event)
 
     pos.x = event.button.x;
     pos.y = event.button.y;
-    if (MouseUp (pos))
-      return false; // (*)
+    if (MouseUp (pos)) // (*)
+      return false;
     break;
   }
 
