@@ -1,7 +1,7 @@
 /*
  * This file is part of the planetblupi source code
  * Copyright (C) 1997, Daniel Roux & EPSITEC SA
- * Copyright (C) 2017, Mathieu Schroeter
+ * Copyright (C) 2017-2018, Mathieu Schroeter
  * http://epsitec.ch; http://www.blupi.org; http://github.com/blupi-games
  *
  * This program is free software: you can redistribute it and/or modify
@@ -193,10 +193,11 @@ CDecor::Init (Sint32 channel, Sint32 icon)
       m_decor[x][y].objectChannel = -1;
       m_decor[x][y].objectIcon    = -1;
 
-      m_decor[x][y].fog       = FOGHIDE; // caché
-      m_decor[x][y].rankMove  = -1;
-      m_decor[x][y].workBlupi = -1;
-      m_decor[x][y].fire      = 0;
+      m_decor[x][y].fog        = FOGHIDE; // caché
+      m_decor[x][y].rankMove   = -1;
+      m_decor[x][y].workBlupi  = -1;
+      m_decor[x][y].fire       = 0;
+      m_decorMem[x][y].flagged = false;
     }
   }
 
@@ -403,6 +404,10 @@ CDecor::PutObject (Point cel, Sint32 channel, Sint32 icon)
   m_decor[cel.x / 2][cel.y / 2].objectChannel = channel;
   m_decor[cel.x / 2][cel.y / 2].objectIcon    = icon;
 
+  /* When flagged, it's possible to build a mine for iron */
+  if (icon == 124)
+    m_decorMem[cel.x / 2][cel.y / 2].flagged = true;
+
   SubDrapeau (cel); // on pourra de nouveau planter un drapeau
 
   return true;
@@ -1272,7 +1277,7 @@ CDecor::Build (Rect clip, Point posMouse)
         rank = m_rankBlupi[x][y];
         if (
           rank != -1 && // un blupi sur cette cellule ?
-          !m_blupi[rank].bCache)
+          !m_blupi[rank].bCache && m_blupi[rank].bExist)
         {
           cel.x = m_blupi[rank].cel.x;
           cel.y = m_blupi[rank].cel.y;
@@ -1835,13 +1840,13 @@ CDecor::CelOkForAction (
         }
       }
 
-      if (action == EV_ACTION_BUILD4) // mine ?
+      GetFloor (cel, channel, icon);
+      if ( // mine ?
+        action == EV_ACTION_BUILD4 &&
+        ((!g_restoreBugs && !m_decorMem[cel.x / 2][cel.y / 2].flagged) || // fixed
+         (g_restoreBugs && (channel != CHFLOOR || icon != 71)))) // funny bug
       {
-        GetFloor (cel, channel, icon);
-        if (channel != CHFLOOR || icon != 71) // terre à fer ?
-        {
-          error = Errors::GROUND; // sol pas adéquat
-        }
+        error = Errors::GROUND; // sol pas adéquat
       }
 
       if (
@@ -3315,8 +3320,8 @@ CDecor::MemoPos (Sint32 rank, bool bRecord)
 {
   Point pos;
 
-  pos.x = LXIMAGE / 2;
-  pos.y = LYIMAGE / 2;
+  pos.x = LXIMAGE () / 2;
+  pos.y = LYIMAGE () / 2;
 
   if (rank < 0 || rank >= 4)
     return;
@@ -3494,3 +3499,9 @@ CDecor::IsUndo ()
 {
   return (m_pUndoDecor != nullptr);
 }
+
+void
+CDecor::InvalidateGrounds ()
+{
+  m_bGroundRedraw = true;
+}
\ No newline at end of file
