@@ -1,7 +1,7 @@
 /*
  * This file is part of the planetblupi source code
  * Copyright (C) 1997, Daniel Roux & EPSITEC SA
- * Copyright (C) 2017, Mathieu Schroeter
+ * Copyright (C) 2017-2018, Mathieu Schroeter
  * http://epsitec.ch; http://www.blupi.org; http://github.com/blupi-games
  *
  * This program is free software: you can redistribute it and/or modify
@@ -807,8 +807,9 @@ CDecor::StatisticDown (Point pos)
 
   StatisticUpdate ();
 
-  hili = StatisticDetect (pos);
-  if (hili < 0)
+  bool disable;
+  hili = StatisticDetect (pos, disable);
+  if (hili < 0 || disable)
     return false;
 
   if (m_bStatUp && hili == 0) // flèche up ?
@@ -817,8 +818,8 @@ CDecor::StatisticDown (Point pos)
     if (m_statFirst < STATNB - 1)
       m_statFirst = 0;
     StatisticUpdate ();
-    pos.x = LXIMAGE / 2;
-    pos.y = LYIMAGE / 2;
+    pos.x = LXIMAGE () / 2;
+    pos.y = LYIMAGE () / 2;
     m_pSound->PlayImage (SOUND_OPEN, pos);
     return true;
   }
@@ -830,8 +831,8 @@ CDecor::StatisticDown (Point pos)
     else
       m_statFirst += STATNB - 2;
     StatisticUpdate ();
-    pos.x = LXIMAGE / 2;
-    pos.y = LYIMAGE / 2;
+    pos.x = LXIMAGE () / 2;
+    pos.y = LYIMAGE () / 2;
     m_pSound->PlayImage (SOUND_OPEN, pos);
     return true;
   }
@@ -984,11 +985,11 @@ select:
 // Souris déplacée dans les statistiques.
 
 bool
-CDecor::StatisticMove (Point pos)
+CDecor::StatisticMove (Point pos, bool & disable)
 {
   Sint32 rank;
 
-  rank = StatisticDetect (pos);
+  rank = StatisticDetect (pos, disable);
 
   if (rank != m_statHili) // autre mise en évidence ?
     m_statHili = rank;
@@ -1007,10 +1008,12 @@ CDecor::StatisticUp (Point pos)
 // Détecte dans quelle statistique est la souris.
 
 Sint32
-CDecor::StatisticDetect (Point pos)
+CDecor::StatisticDetect (Point pos, bool & disable)
 {
   Sint32 rank;
 
+  disable = false;
+
   // Dans un bouton stop/setup/write ?
   if (pos.x >= 10 && pos.x <= 10 + 42 * 3 && pos.y >= 422 && pos.y <= 422 + 40)
   {
@@ -1029,12 +1032,18 @@ CDecor::StatisticDetect (Point pos)
     if (rank >= STATNB)
       return -1;
 
-    auto pStatistic = StatisticGet (rank);
+    if (rank == 11 && this->m_bStatDown)
+      return rank;
+
+    if (rank == 0 && this->m_bStatUp)
+      return rank;
+
+    auto pStatistic = StatisticGet (m_statFirst + rank - !!m_bStatUp);
     if (
       this->GetSkill () >= 1 && pStatistic->perso >= 0 &&
       (pStatistic->perso != 0 && pStatistic->perso != 8))
     {
-      return -1;
+      disable = true;
     }
 
     return rank;
