@@ -1,7 +1,7 @@
 /*
  * This file is part of the planetblupi source code
  * Copyright (C) 1997, Daniel Roux & EPSITEC SA
- * Copyright (C) 2017, Mathieu Schroeter
+ * Copyright (C) 2017-2018, Mathieu Schroeter
  * http://epsitec.ch; http://www.blupi.org; http://github.com/blupi-games
  *
  * This program is free software: you can redistribute it and/or modify
@@ -20,18 +20,16 @@
 
 #pragma once
 
-#include <SDL2/SDL_stdinc.h>
+#include <SDL_stdinc.h>
+
+#include "display.h"
 
 // clang-format off
 #define _INTRO          true    // true for init screen
 
-#define SCRFACTOR       4 / 3
-#define LXIMAGE         (480 * SCRFACTOR + (480 * SCRFACTOR) % 2) // window size
-#define LYIMAGE         480
-
 #define POSDRAWX        144     // draw surface
 #define POSDRAWY        15
-#define DIMDRAWX        (LXIMAGE - (640 - 480))
+#define DIMDRAWX        (LXIMAGE () - (LXLOGIC () - LYLOGIC ()))
 #define DIMDRAWY        450
 
 #define POSMAPX         8       // map surface
@@ -559,6 +557,8 @@ enum ShiftDirection {
 #define EV_READ8                (EV_OFFSET+308)
 #define EV_READ9                (EV_OFFSET+309)
 
+#define EV_READ_EXIT            (EV_OFFSET+320)
+
 #define EV_WRITE0               (EV_OFFSET+310)
 #define EV_WRITE1               (EV_OFFSET+311)
 #define EV_WRITE2               (EV_OFFSET+312)
@@ -611,6 +611,8 @@ enum ShiftDirection {
 
 #define EV_MUSIC_STOP           (EV_OFFSET+550)
 
+#define EV_SETUP_EXIT           (EV_OFFSET+560)
+
 #define EV_PREV                 (EV_OFFSET+600)
 #define EV_NEXT                 (EV_OFFSET+601)
 #define EV_MOVIE                (EV_OFFSET+602)
