@@ -24,9 +24,9 @@
 #include <unordered_map>
 #include <vector>
 
-#include "blupi.h"
 #include "button.h"
 #include "menu.h"
+#include "pixmap.h"
 #include "progress.h"
 
 class CMovie;
@@ -42,10 +42,12 @@ typedef struct {
 } Button;
 
 typedef struct {
-  Uint32 phase;
-  char   backName[20];
-  Sint32 bCDrom;
-  Button buttons[MAXBUTTON];
+  Uint32        phase;
+  char          backName[20];
+  std::string   backWideName;
+  CPixmap::Mode mode;
+  Sint32        bCDrom;
+  Button        buttons[MAXBUTTON];
 } Phase;
 
 typedef struct {
@@ -92,16 +94,18 @@ public:
   CEvent ();
   ~CEvent ();
 
+  bool  IsDemoPlaying ();
   Point GetMousePos ();
   void
               Create (CPixmap * pPixmap, CDecor * pDecor, CSound * pSound, CMovie * pMovie);
-  void        SetFullScreen (bool bFullScreen);
+  void        SetFullScreen (bool bFullScreen, double prevScale = 1);
   Sint32      GetWorld ();
   Sint32      GetPhysicalWorld ();
   Sint32      GetImageWorld ();
   bool        IsHelpHide ();
   bool        IsBaseMusicAvailable (Sint32 music, const std::string & format);
   std::string GetMusicLocation (Sint32 music);
+  bool        LoadBackground ();
   bool        ChangePhase (Uint32 phase);
   bool        MovieToStart ();
   Uint32      GetPhase ();
@@ -147,24 +151,25 @@ public:
   void SetWindowSize (Uint8 newScale);
   void SetUpdateVersion (const std::string & version);
 
+  bool EventButtons (const SDL_Event & event, Point pos);
+  bool PlayMove (Point pos);
+
   static void PushUserEvent (Sint32 code, void * data = nullptr);
 
 protected:
   void DrawTextCenter (const char * text, Sint32 x, Sint32 y, Sint32 font = 0);
-  bool CreateButtons ();
-  bool EventButtons (const SDL_Event & event, Point pos);
+  bool CreateButtons (Sint32 phase);
   bool MouseOnButton (Point pos);
   Sint32 SearchPhase (Uint32 phase);
   void   DecorShift (Sint32 dx, Sint32 dy);
 
   bool PlayDown (Point pos, const SDL_Event & event);
-  bool PlayMove (Point pos);
   bool PlayUp (Point pos);
 
   Language GetStartLanguage ();
   Language GetLanguage ();
   void     SetLanguage (Language lang = Language::undef);
-  void     SetWindowSize (Uint8 prevScale, Uint8 newScale);
+  void     SetWindowSize (double prevScale, double newScale);
 
   void ChangeButtons (Sint32 message);
 
@@ -241,6 +246,7 @@ protected:
   bool                                     m_bHelp;
   bool                                     m_bAllMissions;
   Sint32                                   m_scrollSpeed;
+  Sint32                                   m_scrollSpeedPrev;
   bool                                     m_bPause;
   bool                                     m_bShift;
   Sint32                                   m_shiftPhase;
@@ -269,6 +275,7 @@ protected:
   Sint32                                   m_updateBlinking;
   std::string                              m_updateVersion;
   Uint32                                   shiftDirection;
+  bool                                     statDisabled;
 };
 
 /////////////////////////////////////////////////////////////////////////////
