@@ -1,7 +1,7 @@
 /*
  * This file is part of the planetblupi source code
  * Copyright (C) 1997, Daniel Roux & EPSITEC SA
- * Copyright (C) 2017, Mathieu Schroeter
+ * Copyright (C) 2017-2018, Mathieu Schroeter
  * http://epsitec.ch; http://www.blupi.org; http://github.com/blupi-games
  *
  * This program is free software: you can redistribute it and/or modify
@@ -185,7 +185,9 @@ CMenu::Create (
     pos = m_pos;
     pos.x += DIMBUTTONX / 2;
     pos.y += DIMBUTTONY / 2;
-    SDL_WarpMouseInWindow (g_window, pos.x * g_zoom, pos.y * g_zoom);
+
+    this->m_pPixmap->FromGameToDisplay (pos.x, pos.y);
+    SDL_WarpMouseInWindow (g_window, pos.x, pos.y);
   }
 
   m_selRank = Detect (pos);
